// Generated by CoffeeScript 1.12.2
(function() {
  module.exports = {
    chipArray: function(pattern, housing) {
      var d, heels, ipc, leadToPad, pad, params, ref, ref1, ref2, settings, sides, toes;
      settings = pattern.settings;
      if (housing.leadSpan == null) {
        housing.leadSpan = housing.bodyWidth;
      }
      toes = {
        M: 0.55,
        N: 0.45,
        L: 0.35
      };
      heels = {
        M: -0.05,
        N: -0.07,
        L: -0.1
      };
      sides = {
        M: -0.05,
        N: -0.07,
        L: -0.1
      };
      params = this._params(pattern, housing);
      params.Jt = (ref = pattern.toe) != null ? ref : toes[settings.densityLevel];
      params.Jh = (ref1 = pattern.heel) != null ? ref1 : heels[settings.densityLevel];
      params.Js = (ref2 = pattern.side) != null ? ref2 : sides[settings.densityLevel];
      params.Lmin = housing.leadSpan.min;
      params.Lmax = housing.leadSpan.max;
      ipc = this._ipc7351(params);
      ipc.clearance = settings.clearance.padToPad;
      ipc.pitch = housing.pitch;
      pad = this._pad(ipc, pattern);
      pad.courtyard = params.courtyard;
      pad = this._choosePreferred(pad, pattern, housing);
      leadToPad = (pad.distance + pad.width - housing.leadSpan.nom) / 2;
      if (leadToPad < settings.minimum.spaceForIron) {
        d = settings.minimum.spaceForIron - leadToPad;
        pad.width += d;
        pad.distance += d;
      }
      return pad;
    },
    cornerConcave: function(pattern, housing) {
      var d, inPeriph, leadToPad1, leadToPad2, outPeriph, pad, params, settings;
      settings = pattern.settings;
      outPeriph = {
        M: 0.35,
        N: 0.25,
        L: 0.15
      }[settings.densityLevel];
      inPeriph = {
        M: 0.1,
        N: 0.0,
        L: -0.05
      }[settings.densityLevel];
      params = this._params(pattern, housing);
      pad = {
        width: params.Tmax + outPeriph + inPeriph,
        height: params.Wmax + outPeriph + inPeriph,
        distance1: housing.rowSpan.nom - params.Tmax + outPeriph / 2 - inPeriph / 2,
        distance2: housing.columnSpan.nom - params.Wmax + outPeriph / 2 - inPeriph / 2,
        courtyard: params.courtyard
      };
      pad = this._choosePreferred(pad, pattern, housing);
      if (settings.preferManufacturer) {
        if (housing.padSpace1 != null) {
          pad.distance1 = housing.padSpace1 + housing.padWidth;
        }
        if (housing.padSpan1 != null) {
          pad.distance1 = housing.padSpan1 - housing.padWidth;
        }
        if (housing.padSpace2 != null) {
          pad.distance2 = housing.padSpace2 + housing.padHeight;
        }
        if (housing.padSpan2 != null) {
          pad.distance2 = housing.padSpan2 - housing.padHeight;
        }
      }
      leadToPad1 = (pad.distance1 + pad.width - housing.rowSpan.nom) / 2;
      leadToPad2 = (pad.distance2 + pad.height - housing.columnSpan.nom) / 2;
      if (leadToPad1 < settings.minimum.spaceForIron) {
        d = settings.minimum.spaceForIron - leadToPad1;
        pad.width += d;
        pad.distance1 += d;
      }
      if (leadToPad2 < settings.minimum.spaceForIron) {
        d = settings.minimum.spaceForIron - leadToPad2;
        pad.height += d;
        pad.distance2 += d;
      }
      return pad;
    },
    dual: function(pattern, housing, option) {
      var d, ipc, leadToPad, pad, params, settings;
      settings = pattern.settings;
      switch (option) {
        case 'flatlead':
          params = this._flatlead(pattern, housing);
          break;
        case 'soj':
          params = this._jlead(pattern, housing);
          break;
        case 'sol':
          params = this._llead(pattern, housing);
          break;
        case 'sop':
          params = this._gullwing(pattern, housing);
      }
      params.Lmin = housing.leadSpan.min;
      params.Lmax = housing.leadSpan.max;
      ipc = this._ipc7351(params);
      ipc.clearance = settings.clearance.padToPad;
      ipc.pitch = housing.pitch;
      if (option === 'sop') {
        ipc.body = housing.bodyWidth.nom;
      }
      pad = this._pad(ipc, pattern);
      pad.courtyard = params.courtyard;
      pad = this._choosePreferred(pad, pattern, housing);
      leadToPad = (pad.distance + pad.width - housing.leadSpan.nom) / 2;
      if (leadToPad < settings.minimum.spaceForIron) {
        d = settings.minimum.spaceForIron - leadToPad;
        pad.width += d;
        pad.distance += d;
      }
      return pad;
    },
    gridArray: function(pattern, housing, option) {
      var adj, clearance, courtyard, horizontalPitch, pad, padDiameter, padHeight, padWidth, pitch, ref, ref1, ref2, ref3, ref4, roundOff, settings, verticalPitch;
      settings = pattern.settings;
      switch (option) {
        case 'bga':
          adj = settings.ball.collapsible ? 0.8 : 1;
          padDiameter = housing.leadDiameter.nom * adj;
          roundOff = 0.01;
          padDiameter = Math.round(padDiameter / roundOff) * roundOff;
          courtyard = housing.pitch * 0.8;
          roundOff = 0.05;
          courtyard = Math.round(courtyard / roundOff) * roundOff;
          break;
        case 'cga':
          padDiameter = housing.leadDiameter.nom + 0.1;
          pitch = (ref = housing.pitch) != null ? ref : Math.min(housing.horizontalPitch, housing.verticalPitch);
          clearance = (ref1 = housing.padSpace) != null ? ref1 : settings.clearance.padToPad;
          if (padDiameter > pitch - clearance) {
            padDiameter = pitch - clearance;
          }
          courtyard = 1;
          break;
        case 'lga':
          clearance = (ref2 = housing.padSpace) != null ? ref2 : settings.clearance.padToPad;
          padWidth = housing.leadLength.nom + 0.1;
          horizontalPitch = (ref3 = housing.horizontalPitch) != null ? ref3 : housing.pitch;
          if (padWidth > horizontalPitch - clearance) {
            padWidth = horizontalPitch - clearance;
          }
          padHeight = housing.leadWidth.nom + 0.1;
          verticalPitch = (ref4 = housing.verticalPitch) != null ? ref4 : housing.pitch;
          if (padHeight > verticalPitch - clearance) {
            padHeight = verticalPitch - clearance;
          }
          courtyard = 1;
      }
      if (padWidth == null) {
        padWidth = padDiameter;
      }
      if (padHeight == null) {
        padHeight = padDiameter;
      }
      pad = {
        width: padWidth,
        height: padHeight,
        courtyard: courtyard
      };
      return this._choosePreferred(pad, pattern, housing);
    },
    padDiameter: function(pattern, housing, holeDiameter) {
      var clearance, padDiameter, pitch, ref, ref1, ref2, settings, sizeRoundoff;
      settings = pattern.settings;
      padDiameter = holeDiameter * settings.ratio.padToHole;
      if (padDiameter < (holeDiameter + 2 * settings.minimum.ringWidth)) {
        padDiameter = holeDiameter + 2 * settings.minimum.ringWidth;
      }
      if ((housing.pitch != null) || ((housing.horizontalPitch != null) && (housing.verticalPitch != null))) {
        pitch = (ref = housing.pitch) != null ? ref : Math.min(Math.abs(housing.horizontalPitch), Math.abs(housing.verticalPitch));
        clearance = (ref1 = housing.padSpace) != null ? ref1 : settings.clearance.padToPad;
        if (padDiameter > pitch - clearance) {
          padDiameter = pitch - clearance;
        }
      }
      sizeRoundoff = (ref2 = pattern.sizeRoundoff) != null ? ref2 : 0.05;
      return Math.round(padDiameter / sizeRoundoff) * sizeRoundoff;
    },
    pak: function(pattern, housing) {
      var d, leadIpc, leadPad, leadToPad1, leadToPad2, pad, params, ref, ref1, ref2, ref3, settings, tabIpc, tabPad;
      settings = pattern.settings;
      params = this._gullwing(pattern, housing);
      params.Lmin = housing.leadSpan.min;
      params.Lmax = housing.leadSpan.max;
      leadIpc = this._ipc7351(params);
      leadIpc.clearance = settings.clearance.padToPad;
      leadIpc.pitch = housing.pitch;
      leadPad = this._pad(leadIpc, pattern);
      params.Tmin = (ref = housing.tabLength.min) != null ? ref : housing.tabLength;
      params.Tmax = (ref1 = housing.tabLength.max) != null ? ref1 : housing.tabLength;
      params.Wmin = (ref2 = housing.tabWidth.min) != null ? ref2 : housing.tabWidth;
      params.Wmax = (ref3 = housing.tabWidth.max) != null ? ref3 : housing.tabWidth;
      tabIpc = this._ipc7351(params);
      tabPad = this._pad(tabIpc, pattern);
      pad = {
        width1: leadPad.width,
        height1: leadPad.height,
        distance1: leadPad.distance,
        width2: tabPad.width,
        height2: tabPad.height,
        distance2: tabPad.distance,
        courtyard: params.courtyard
      };
      pad = this._choosePreferred(pad, pattern, housing);
      leadToPad1 = (pad.distance1 + pad.width1 - housing.leadSpan.nom) / 2;
      leadToPad2 = (pad.distance2 + pad.width2 - housing.leadSpan.nom) / 2;
      if (leadToPad1 < settings.minimum.spaceForIron) {
        d = settings.minimum.spaceForIron - leadToPad1;
        pad.width1 += d;
        pad.distance1 += d;
      }
      if (leadToPad2 < settings.minimum.spaceForIron) {
        d = settings.minimum.spaceForIron - leadToPad2;
        pad.width2 += d;
        pad.distance2 += d;
      }
      return pad;
    },
    quad: function(pattern, housing, option) {
      var columnIpc, columnPad, columnSpan, d, leadToPad1, leadToPad2, pad, params, rowIpc, rowPad, rowSpan, settings;
      settings = pattern.settings;
      switch (option) {
        case 'qfn':
          params = this._nolead(pattern, housing);
          params.Lmin = housing.bodyWidth.min;
          params.Lmax = housing.bodyWidth.max;
          if (housing.pullBack != null) {
            params.Lmin -= 2 * housing.pullBack.nom;
            params.Lmax -= 2 * housing.pullBack.nom;
          }
          rowIpc = this._ipc7351(params);
          params.Lmin = housing.bodyLength.min;
          params.Lmax = housing.bodyLength.max;
          if (housing.pullBack != null) {
            params.Lmin -= 2 * housing.pullBack.nom;
            params.Lmax -= 2 * housing.pullBack.nom;
          }
          columnIpc = this._ipc7351(params);
          rowSpan = housing.bodyWidth.nom;
          columnSpan = housing.bodyLength.nom;
          break;
        case 'qfp':
          params = this._gullwing(pattern, housing);
          params.Lmin = housing.rowSpan.min;
          params.Lmax = housing.rowSpan.max;
          rowIpc = this._ipc7351(params);
          params.Lmin = housing.columnSpan.min;
          params.Lmax = housing.columnSpan.max;
          columnIpc = this._ipc7351(params);
          rowSpan = housing.rowSpan.nom;
          columnSpan = housing.columnSpan.nom;
      }
      rowIpc.clearance = settings.clearance.padToPad;
      rowIpc.pitch = housing.pitch;
      if (option === 'qfp') {
        rowIpc.body = housing.bodyWidth.nom;
      }
      rowPad = this._pad(rowIpc, pattern);
      columnIpc.clearance = settings.clearance.padToPad;
      columnIpc.pitch = housing.pitch;
      if (option === 'qfp') {
        columnIpc.body = housing.bodyLength.nom;
      }
      columnPad = this._pad(columnIpc, pattern);
      pad = {
        width1: rowPad.width,
        height1: rowPad.height,
        distance1: rowPad.distance,
        width2: columnPad.width,
        height2: columnPad.height,
        distance2: columnPad.distance,
        trimmed: rowPad.trimmed || columnPad.trimmed,
        courtyard: params.courtyard
      };
      pad = this._choosePreferred(pad, pattern, housing);
      if (settings.preferManufacturer) {
        if (housing.padWidth != null) {
          if (housing.padSpace1 != null) {
            pad.distance1 = housing.padSpace1 + housing.padWidth;
          }
          if (housing.padSpan1 != null) {
            pad.distance1 = housing.padSpan1 - housing.padWidth;
          }
          if (housing.padSpace2 != null) {
            pad.distance2 = housing.padSpace2 + housing.padWidth;
          }
          if (housing.padSpan2 != null) {
            pad.distance2 = housing.padSpan2 - housing.padWidth;
          }
        }
      }
      leadToPad1 = (pad.distance1 + pad.width1 - rowSpan) / 2;
      leadToPad2 = (pad.distance2 + pad.width2 - columnSpan) / 2;
      if (leadToPad1 < settings.minimum.spaceForIron) {
        d = settings.minimum.spaceForIron - leadToPad1;
        pad.width1 += d;
        pad.distance1 += d;
        leadToPad1 = (pad.distance1 + pad.width1 - rowSpan) / 2;
      }
      if (leadToPad2 < settings.minimum.spaceForIron) {
        d = settings.minimum.spaceForIron - leadToPad2;
        pad.width2 += d;
        pad.distance2 += d;
      }
      return pad;
    },
    son: function(pattern, housing) {
      var d, dw, ipc, leadToPad, pad, params, settings, space;
      settings = pattern.settings;
      params = this._nolead(pattern, housing);
      params.Lmin = housing.bodyWidth.min;
      params.Lmax = housing.bodyWidth.max;
      if (housing.pullBack != null) {
        params.Lmin -= 2 * housing.pullBack.nom;
        params.Lmax -= 2 * housing.pullBack.nom;
      }
      ipc = this._ipc7351(params);
      ipc.clearance = settings.clearance.padToPad;
      ipc.pitch = housing.pitch;
      pad = this._pad(ipc, pattern);
      if (housing.leadLength1 != null) {
        dw = housing.leadLength1.nom - housing.leadLength.nom;
        space = pad.distance - pad.width;
        if ((space - dw) < settings.clearance.padToPad) {
          dw = space - settings.clearance.padToPad;
        }
        pad.width1 = pad.width + dw;
      }
      pad.courtyard = params.courtyard;
      pad = this._choosePreferred(pad, pattern, housing);
      leadToPad = (pad.distance + pad.width - housing.bodyWidth.nom) / 2;
      if (leadToPad < settings.minimum.spaceForIron) {
        d = settings.minimum.spaceForIron - leadToPad;
        pad.width += d;
        pad.distance += d;
      }
      return pad;
    },
    sot: function(pattern, housing) {
      var d, ipc1, ipc2, leadToPad, pad, pad1, pad2, params, settings;
      settings = pattern.settings;
      housing.leadWidth = housing.leadWidth1;
      if (housing.flatlead) {
        params = this._flatlead(pattern, housing);
      } else {
        params = this._gullwing(pattern, housing);
      }
      params.Lmin = housing.leadSpan.min;
      params.Lmax = housing.leadSpan.max;
      params.body = housing.bodyWidth;
      ipc1 = this._ipc7351(params);
      ipc1.clearance = settings.clearance.padToPad;
      ipc1.pitch = housing.pitch;
      ipc1.body = housing.bodyWidth.nom;
      pad1 = this._pad(ipc1, pattern, housing.pitch);
      params.Wmin = housing.leadWidth2.min;
      params.Wmax = housing.leadWidth2.max;
      ipc2 = this._ipc7351(params);
      ipc2.body = housing.bodyWidth.nom;
      pad2 = this._pad(ipc2, pattern);
      pad = {
        width1: pad1.width,
        height1: pad1.height,
        distance: pad1.distance,
        width2: pad2.width,
        height2: pad2.height,
        courtyard: params.courtyard,
        trimmed: pad1.trimmed || pad2.trimmed
      };
      pad = this._choosePreferred(pad, pattern, housing);
      leadToPad = (pad.distance + pad.width1 / 2 + pad.width2 / 2 - housing.leadSpan.nom) / 2;
      if (leadToPad < settings.minimum.spaceForIron) {
        d = settings.minimum.spaceForIron - leadToPad;
        pad.width1 += d;
        pad.width2 += d;
        pad.distance += d;
      }
      return pad;
    },
    throughHole: function(pattern, housing) {
      var diameter, h, hole, pad, padDiameter, ref, settings, sizeRoundoff, w;
      settings = pattern.settings;
      if (housing.leadDiameter != null) {
        diameter = housing.leadDiameter.max;
      } else {
        w = housing.leadWidth.max;
        h = housing.leadHeight.max;
        diameter = Math.sqrt(w * w + h * h);
      }
      hole = diameter + 2 * settings.clearance.leadToHole;
      if (hole < settings.minimum.holeDiameter) {
        hole = settings.minimum.holeDiameter;
      }
      sizeRoundoff = (ref = pattern.sizeRoundoff) != null ? ref : 0.05;
      hole = Math.ceil(hole / sizeRoundoff) * sizeRoundoff;
      padDiameter = this.padDiameter(pattern, housing, hole);
      pad = {
        hole: hole,
        width: padDiameter,
        height: padDiameter
      };
      return this._choosePreferred(pad, pattern, housing);
    },
    twoPin: function(pattern, housing, option) {
      var courtyard, d, heel, heels, height, ipc, leadToPad, pad, params, ref, ref1, ref2, ref3, settings, side, sides, toe, toes;
      if (option == null) {
        option = 'chip';
      }
      if (option !== 'radial') {
        if (housing.bodyWidth == null) {
          housing.bodyWidth = housing.bodyDiameter;
        }
        if (housing.leadWidth == null) {
          housing.leadWidth = housing.bodyWidth;
        }
        if (housing.leadSpan == null) {
          housing.leadSpan = housing.bodyLength;
        }
      }
      settings = pattern.settings;
      switch (option) {
        case 'chip':
          height = (ref = housing.height.nom) != null ? ref : housing.height;
          toe = height * 0.5;
          heel = height * 0.1;
          side = height * 0.15;
          courtyard = height * 0.4;
          courtyard = Math.round(courtyard / 0.01) * 0.01;
          if (courtyard > 0.25) {
            courtyard = 0.25;
          }
          break;
        case 'concave':
          toes = {
            M: 0.55,
            N: 0.45,
            L: 0.35
          };
          heels = {
            M: -0.05,
            N: -0.07,
            L: -0.1
          };
          sides = {
            M: -0.05,
            N: -0.07,
            L: -0.1
          };
          break;
        case 'crystal':
          toes = height >= 10 ? {
            M: 1,
            N: 0.7,
            L: 0.4
          } : {
            M: 0.7,
            N: 0.5,
            L: 0.3
          };
          heels = height >= 10 ? {
            M: 0,
            N: -0.05,
            L: -0.1
          } : {
            M: 0,
            N: -0.1,
            L: -0.2
          };
          sides = height >= 10 ? {
            M: 0.6,
            N: 0.5,
            L: 0.4
          } : {
            M: 0.5,
            N: 0.4,
            L: 0.3
          };
          courtyard = {
            M: 1,
            N: 0.5,
            L: 0.25
          }[settings.densityLevel];
          break;
        case 'dfn':
          toes = {
            M: 0.6,
            N: 0.4,
            L: 0.2
          };
          heels = {
            M: 0.2,
            N: 0.1,
            L: 0.02
          };
          sides = {
            M: 0.1,
            N: 0.05,
            L: 0.01
          };
          break;
        case 'melf':
          toes = {
            M: 0.6,
            N: 0.4,
            L: 0.2
          };
          heels = {
            M: 0.2,
            N: 0.1,
            L: 0.02
          };
          sides = {
            M: 0.1,
            N: 0.05,
            L: 0.01
          };
          break;
        case 'molded':
          toes = {
            M: 0.25,
            N: 0.15,
            L: 0.07
          };
          heels = {
            M: 0.8,
            N: 0.5,
            L: 0.2
          };
          sides = {
            M: 0.01,
            N: -0.05,
            L: -0.1
          };
          break;
        case 'radial':
          pad = this.throughHole(pattern, housing);
          pad.distance = housing.leadSpan.nom;
          pad.courtyard = {
            M: 0.5,
            N: 0.25,
            L: 0.12
          }[settings.densityLevel];
          return pad;
        case 'sod':
          toes = {
            M: 0.55,
            N: 0.35,
            L: 0.15
          };
          heels = {
            M: 0.45,
            N: 0.35,
            L: 0.25
          };
          sides = {
            M: 0.05,
            N: 0.03,
            L: 0.01
          };
          break;
        case 'sodfl':
          toes = {
            M: 0.3,
            N: 0.2,
            L: 0.1
          };
          heels = {
            M: 0,
            N: 0,
            L: 0
          };
          sides = {
            M: 0.05,
            N: 0,
            L: -0.05
          };
          courtyard = {
            M: 0.2,
            N: 0.15,
            L: 0.12
          }[settings.densityLevel];
      }
      if (toe == null) {
        toe = toes[settings.densityLevel];
      }
      if (heel == null) {
        heel = heels[settings.densityLevel];
      }
      if (side == null) {
        side = sides[settings.densityLevel];
      }
      params = this._params(pattern, housing);
      params.Jt = (ref1 = pattern.toe) != null ? ref1 : toe;
      params.Jh = (ref2 = pattern.heel) != null ? ref2 : heel;
      params.Js = (ref3 = pattern.side) != null ? ref3 : side;
      params.Lmin = housing.leadSpan.min;
      params.Lmax = housing.leadSpan.max;
      ipc = this._ipc7351(params);
      ipc.clearance = settings.clearance.padToPad;
      pad = this._pad(ipc, pattern);
      pad.courtyard = courtyard != null ? courtyard : params.courtyard;
      pad = this._choosePreferred(pad, pattern, housing);
      leadToPad = (pad.distance + pad.width - housing.leadSpan.nom) / 2;
      if (leadToPad < settings.minimum.spaceForIron) {
        d = settings.minimum.spaceForIron - leadToPad;
        pad.width += d;
        pad.distance += d;
      }
      return pad;
    },
    _choosePreferred: function(pad, pattern, housing) {
      var settings;
      settings = pattern.settings;
      if (settings.preferManufacturer) {
        if (housing.padWidth != null) {
          pad.width = housing.padWidth;
          if (housing.padSpace != null) {
            pad.distance = housing.padSpace + housing.padWidth;
          }
          if (housing.padSpan != null) {
            pad.distance = housing.padSpan - housing.padWidth;
          }
        }
        if (housing.padWidth1 != null) {
          pad.width1 = housing.padWidth1;
          if (housing.padSpace != null) {
            pad.distance1 = housing.padSpace + housing.padWidth1;
          }
          if (housing.padSpace1 != null) {
            pad.distance1 = housing.padSpace1 + housing.padWidth1;
          }
          if (housing.padSpan != null) {
            pad.distance1 = housing.padSpan - housing.padWidth1;
          }
          if (housing.padSpan1 != null) {
            pad.distance1 = housing.padSpan1 - housing.padWidth1;
          }
        }
        if (housing.padWidth2 != null) {
          pad.width2 = housing.padWidth2;
          if (housing.padSpace != null) {
            pad.distance2 = housing.padSpace + housing.padWidth2;
          }
          if (housing.padSpace2 != null) {
            pad.distance2 = housing.padSpace2 + housing.padWidth2;
          }
          if (housing.padSpan != null) {
            pad.distance2 = housing.padSpan - housing.padWidth2;
          }
          if (housing.padSpan2 != null) {
            pad.distance2 = housing.padSpan2 - housing.padWidth2;
          }
        }
        if (housing.padHeight != null) {
          pad.height = housing.padHeight;
        }
        if (housing.padHeight1 != null) {
          pad.height1 = housing.padHeight1;
        }
        if (housing.padHeight2 != null) {
          pad.height2 = housing.padHeight2;
        }
        if (housing.padDistance != null) {
          pad.distance = housing.padDistance;
        }
        if (housing.padDistance1 != null) {
          pad.distance1 = housing.padDistance1;
        }
        if (housing.padDistance2 != null) {
          pad.distance2 = housing.padDistance2;
        }
        if (housing.holeDiameter != null) {
          pad.hole = housing.holeDiameter;
        }
      }
      return pad;
    },
    _gullwing: function(pattern, housing) {
      var heels, params, ref, ref1, ref2, settings, sides, toes;
      settings = pattern.settings;
      toes = {
        M: 0.55,
        N: 0.35,
        L: 0.15
      };
      heels = {
        M: 0.45,
        N: 0.35,
        L: 0.25
      };
      sides = housing.pitch > 0.625 ? {
        M: 0.05,
        N: 0.03,
        L: 0.01
      } : {
        M: 0.01,
        N: -0.02,
        L: -0.04
      };
      params = this._params(pattern, housing);
      params.Jt = (ref = pattern.toe) != null ? ref : toes[settings.densityLevel];
      params.Jh = (ref1 = pattern.heel) != null ? ref1 : heels[settings.densityLevel];
      params.Js = (ref2 = pattern.side) != null ? ref2 : sides[settings.densityLevel];
      return params;
    },
    _flatlead: function(pattern, housing) {
      var heels, params, ref, ref1, ref2, settings, sides, toes;
      settings = pattern.settings;
      toes = {
        M: 0.3,
        N: 0.2,
        L: 0.1
      };
      heels = {
        M: 0,
        N: 0,
        L: 0
      };
      sides = {
        M: 0.05,
        N: 0,
        L: -0.05
      };
      params = this._params(pattern, housing);
      params.Jt = (ref = pattern.toe) != null ? ref : toes[settings.densityLevel];
      params.Jh = (ref1 = pattern.heel) != null ? ref1 : heels[settings.densityLevel];
      params.Js = (ref2 = pattern.side) != null ? ref2 : sides[settings.densityLevel];
      return params;
    },
    _ipc7351: function(params) {
      var Cl, Cs, Cw, F, Jh, Js, Jt, Lmax, Lmin, Ltol, P, Smax, SmaxRms, Smin, Stol, StolRms, Tmax, Tmin, Ttol, Wmax, Wmin, Wtol;
      Lmin = params.Lmin;
      Lmax = params.Lmax;
      Ltol = Lmax - Lmin;
      Tmin = params.Tmin;
      Tmax = params.Tmax;
      Ttol = Tmax - Tmin;
      Wmin = params.Wmin;
      Wmax = params.Wmax;
      Wtol = Wmax - Wmin;
      F = params.F;
      P = params.P;
      Jt = params.Jt;
      Jh = params.Jh;
      Js = params.Js;
      Smin = Lmin - 2 * Tmax;
      Smax = Lmax - 2 * Tmin;
      Stol = Ltol + 2 * Ttol;
      StolRms = Math.sqrt(Ltol * Ltol + 2 * Ttol * Ttol);
      SmaxRms = Smax - (Stol - StolRms) / 2;
      Cl = Ltol;
      Cs = StolRms;
      Cw = Wtol;
      return {
        Zmax: Lmin + 2 * Jt + Math.sqrt(Cl * Cl + F * F + P * P),
        Gmin: SmaxRms - 2 * Jh - Math.sqrt(Cs * Cs + F * F + P * P),
        Xmax: Wmin + 2 * Js + Math.sqrt(Cw * Cw + F * F + P * P)
      };
    },
    _jlead: function(pattern, housing) {
      var heels, params, ref, ref1, ref2, settings, sides, toes;
      settings = pattern.settings;
      toes = {
        M: 0.1,
        N: 0,
        L: -0.1
      };
      heels = {
        M: 0.55,
        N: 0.35,
        L: 0.15
      };
      sides = {
        M: 0.05,
        N: 0.03,
        L: 0.01
      };
      params = this._params(pattern, housing);
      params.Jt = (ref = pattern.heel) != null ? ref : heels[settings.densityLevel];
      params.Jh = (ref1 = pattern.toe) != null ? ref1 : toes[settings.densityLevel];
      params.Js = (ref2 = pattern.side) != null ? ref2 : sides[settings.densityLevel];
      return params;
    },
    _llead: function(pattern, housing) {
      var heels, params, ref, ref1, ref2, settings, sides, toes;
      settings = pattern.settings;
      toes = {
        M: 0.1,
        N: 0,
        L: -0.1
      };
      heels = {
        M: 0.55,
        N: 0.35,
        L: 0.15
      };
      sides = {
        M: 0.01,
        N: -0.02,
        L: -0.04
      };
      params = this._params(pattern, housing);
      params.Jt = (ref = pattern.heel) != null ? ref : heels[settings.densityLevel];
      params.Jh = (ref1 = pattern.toe) != null ? ref1 : toes[settings.densityLevel];
      params.Js = (ref2 = pattern.side) != null ? ref2 : sides[settings.densityLevel];
      return params;
    },
    _nolead: function(pattern, housing) {
      var heel, params, ref, ref1, ref2, settings, side, toe;
      settings = pattern.settings;
      toe = housing.pullBack != null ? 0.0 : {
        M: 0.4,
        N: 0.3,
        L: 0.2
      }[settings.densityLevel];
      heel = 0.0;
      side = housing.pullBack != null ? 0.0 : -0.04;
      params = this._params(pattern, housing);
      params.Jt = (ref = pattern.toe) != null ? ref : toe;
      params.Jh = (ref1 = pattern.heel) != null ? ref1 : heel;
      params.Js = (ref2 = pattern.side) != null ? ref2 : side;
      return params;
    },
    _pad: function(ipc, pattern) {
      var gap, padDistance, padHeight, padWidth, placeRoundoff, ref, ref1, sizeRoundoff, span, trimmed;
      padWidth = (ipc.Zmax - ipc.Gmin) / 2;
      padHeight = ipc.Xmax;
      padDistance = (ipc.Zmax + ipc.Gmin) / 2;
      sizeRoundoff = (ref = pattern.sizeRoundoff) != null ? ref : 0.05;
      placeRoundoff = (ref1 = pattern.placeRoundoff) != null ? ref1 : 0.1;
      padWidth = Math.round(padWidth / sizeRoundoff) * sizeRoundoff;
      padHeight = Math.round(padHeight / sizeRoundoff) * sizeRoundoff;
      padDistance = Math.round(padDistance / placeRoundoff) * placeRoundoff;
      gap = padDistance - padWidth;
      span = padDistance + padWidth;
      trimmed = false;
      if ((ipc.clearance != null) && gap < ipc.clearance) {
        gap = ipc.clearance;
        trimmed = true;
      }
      if ((ipc.body != null) && gap < (ipc.body - 0.1)) {
        gap = ipc.body - 0.1;
        trimmed = true;
      }
      if (trimmed) {
        padWidth = (span - gap) / 2;
        padDistance = (span + gap) / 2;
        padDistance = Math.ceil(padDistance / placeRoundoff) * placeRoundoff;
      }
      if ((ipc.pitch != null) && padHeight > (ipc.pitch - ipc.clearance)) {
        padHeight = ipc.pitch - ipc.clearance;
        trimmed = true;
      }
      return {
        width: padWidth,
        height: padHeight,
        distance: padDistance,
        trimmed: trimmed
      };
    },
    _params: function(pattern, housing) {
      var ref, settings;
      settings = pattern.settings;
      return {
        Tmin: housing.leadLength.min,
        Tmax: housing.leadLength.max,
        Wmin: housing.leadWidth.min,
        Wmax: housing.leadWidth.max,
        F: settings.tolerance.fabrication,
        P: settings.tolerance.placement,
        courtyard: (ref = pattern.courtyard) != null ? ref : {
          M: 0.5,
          N: 0.25,
          L: 0.12
        }[settings.densityLevel]
      };
    }
  };

}).call(this);
