// Generated by CoffeeScript 1.12.2
(function() {
  var KicadGenerator, QedaElement, QedaLibrary, fs, log, mkdirp, path, request, yaml;

  fs = require('fs');

  mkdirp = require('mkdirp');

  path = require('path');

  request = require('sync-request');

  yaml = require('js-yaml');

  KicadGenerator = require('./kicad-generator');

  QedaElement = require('./qeda-element');

  log = require('./qeda-log');

  QedaLibrary = (function() {
    function QedaLibrary(config) {
      if (config == null) {
        config = {};
      }
      this.output = 'kicad';
      this.connection = {
        timeout: 5000
      };
      this.symbol = {
        style: 'default',
        gridSize: 2.5,
        factor: 1,
        pitch: 5,
        shortPinNames: false,
        fill: 'foreground',
        fontSize: {
          "default": 2.5,
          refDes: 2.5,
          name: 2.5,
          pin: 2.5
        },
        lineWidth: {
          "default": 0,
          thick: 0.6,
          thin: 0.2
        },
        space: {
          "default": 2,
          pin: 2,
          attribute: 1.5
        }
      };
      this.pattern = {
        style: 'default',
        densityLevel: 'N',
        decimals: 3,
        polarityMark: 'dot',
        preferManufacturer: true,
        tolerance: {
          "default": 0.1,
          fabrication: 0.1,
          placement: 0.1
        },
        clearance: {
          padToSilk: 0.2,
          padToPad: 0.2,
          padToMask: 0.05,
          leadToHole: 0.1
        },
        ratio: {
          padToHole: 1.5
        },
        minimum: {
          ringWidth: 0.2,
          holeDiameter: 0.2,
          maskWidth: 0.2,
          spaceForIron: 0
        },
        lineWidth: {
          "default": 0.2,
          silkscreen: 0.12,
          assembly: 0.1,
          courtyard: 0.05
        },
        fontSize: {
          "default": 1,
          refDes: 1.2,
          value: 1
        },
        ball: {
          collapsible: true
        }
      };
      this.mergeObjects(this, config);
      this.pattern.minimum.drillDiameter = this.pattern.minimum.holeDiameter;
      this.elements = [];
    }

    QedaLibrary.prototype.add = function(element) {
      var alias, aliases, base1, elements, j, l, len, len1, name, newElement, obj, objs, ref, suffixes;
      objs = this.load(element);
      elements = [];
      for (j = 0, len = objs.length; j < len; j++) {
        obj = objs[j];
        if (obj.abstract) {
          console.warn("'" + element + "': Cannot add abstract component, use it only as base for others");
          return elements;
        }
        if ((base1 = obj.housing).suffix == null) {
          base1.suffix = '';
        }
        suffixes = obj.housing.suffix.replace(/\s+/g, '').split(',');
        name = obj.name;
        obj.name = name + suffixes[0];
        aliases = [];
        if (obj.alias != null) {
          ref = obj.alias.replace(/\s+/g, '').split(',');
          for (l = 0, len1 = ref.length; l < len1; l++) {
            alias = ref[l];
            aliases = aliases.concat(suffixes.map((function(_this) {
              return function(v) {
                return alias + v;
              };
            })(this)));
          }
        }
        if (suffixes.length > 1) {
          aliases = aliases.concat(suffixes.slice(1).map((function(_this) {
            return function(v) {
              return name + v;
            };
          })(this)));
        }
        obj.aliases = aliases;
        newElement = new QedaElement(this, obj);
        this.elements.push(newElement);
        elements.push(newElement);
      }
      return elements;
    };

    QedaLibrary.prototype.addSymbol = function(symbol, name) {
      var def, i, j, newElement, ref, schematic, words;
      schematic = {
        symbol: symbol
      };
      words = name.split('/');
      if (words.length > 1) {
        for (i = j = 0, ref = words.length - 2; 0 <= ref ? j <= ref : j >= ref; i = 0 <= ref ? ++j : --j) {
          schematic[words[i].toLowerCase()] = true;
        }
        name = words[words.length - 1];
      }
      def = {
        name: name,
        schematic: schematic
      };
      newElement = new QedaElement(this, def);
      this.elements.push(newElement);
      return newElement;
    };

    QedaLibrary.prototype.check = function(obj) {};

    QedaLibrary.prototype.generate = function(name) {
      var generator;
      log.start("Render library '" + name + "'");
      this.render();
      log.ok();
      generator = null;
      switch (this.output) {
        case 'kicad':
          generator = new KicadGenerator(this);
      }
      if (generator) {
        log.start("Generate output for '" + name + "'");
        generator.generate(name);
        return log.ok();
      }
    };

    QedaLibrary.prototype.ground = function(name) {
      return this.addSymbol('ground', name);
    };

    QedaLibrary.prototype.load = function(element, force) {
      var j, k, len, modifier, obj, objs, param, ref, v, varObj, variation, variations;
      if (force == null) {
        force = false;
      }
      obj = this.loadYaml(element, force);
      objs = [];
      if (obj.variations != null) {
        variations = obj.variations.replace(/\s+/g, '').toLowerCase().split(',');
        delete obj.variations;
        for (j = 0, len = variations.length; j < len; j++) {
          variation = variations[j];
          if ((typeof filterVariation !== "undefined" && filterVariation !== null) && (variation !== filterVariation)) {
            continue;
          }
          varObj = {};
          this.mergeObjects(varObj, obj);
          for (k in varObj) {
            v = varObj[k];
            ref = k.replace(/\s+/g, '').split('@'), param = ref[0], modifier = ref[1];
            if (modifier != null) {
              if (modifier.toLowerCase() === variation) {
                varObj[param] = v;
              }
              delete varObj[k];
            }
          }
          objs.push(varObj);
        }
      } else {
        objs.push(obj);
      }
      return objs;
    };

    QedaLibrary.prototype.loadYaml = function(element, force) {
      var base, baseElement, baseObj, bases, elementName, elementYaml, error, exclusion, exclusions, filterVariation, j, l, len, len1, localFile, obj, ref, res;
      if (force == null) {
        force = false;
      }
      element = element.toLowerCase();
      ref = element.split('@'), elementName = ref[0], filterVariation = ref[1];
      elementYaml = elementName + '.yaml';
      localFile = './library/' + elementYaml;
      if ((!fs.existsSync(localFile)) || force) {
        log.start("Load '" + elementName + "'");
        elementYaml = elementYaml.split('/').map(function(v) {
          return encodeURIComponent(v);
        }).join('/');
        try {
          res = request('GET', 'https://raw.githubusercontent.com/qeda/library/master/' + elementYaml, {
            timeout: this.connection.timeout
          });
        } catch (error1) {
          error = error1;
          log.error(error.message);
        }
        if (res.statusCode === 200) {
          mkdirp.sync(path.dirname(localFile));
          fs.writeFileSync(localFile, res.body);
          log.ok();
        } else {
          log.warning(res.statusCode);
        }
      }
      log.start("Read '" + elementName + "'");
      try {
        obj = yaml.safeLoad(fs.readFileSync(localFile));
      } catch (error1) {
        error = error1;
        log.error("" + error.message);
      }
      log.ok();
      this.check(obj);
      if (obj.base != null) {
        if (typeof obj.base !== 'string') {
          obj.base = obj.base.toString();
        }
        bases = obj.base.replace(/\s+/g, '').split(',');
        delete obj.base;
        exclusions = ['abstract', 'alias'];
        for (j = 0, len = bases.length; j < len; j++) {
          base = bases[j];
          baseElement = base;
          if (path.dirname(baseElement) === '.') {
            baseElement = path.dirname(element) + '/' + baseElement;
          }
          baseObj = this.loadYaml(baseElement, force);
          for (l = 0, len1 = exclusions.length; l < len1; l++) {
            exclusion = exclusions[l];
            if (baseObj[exclusion] != null) {
              delete baseObj[exclusion];
            }
          }
          this.mergeObjects(baseObj, obj);
          obj = baseObj;
        }
      }
      if (element.indexOf('/') !== -1) {
        if (obj.group == null) {
          obj.group = element.substr(0, element.indexOf('/')).toUpperCase();
        }
      }
      return obj;
    };

    QedaLibrary.prototype.mergeObjects = function(dest, src) {
      var k, results, v;
      results = [];
      for (k in src) {
        v = src[k];
        if (typeof v === 'object' && (!Array.isArray(v)) && dest.hasOwnProperty(k)) {
          results.push(this.mergeObjects(dest[k], v));
        } else {
          results.push(dest[k] = v);
        }
      }
      return results;
    };

    QedaLibrary.prototype.power = function(name) {
      return this.addSymbol('power', name);
    };

    QedaLibrary.prototype.render = function() {
      var baseName, element, i, j, keys, len, name, patterns, ref;
      if (this._rendered == null) {
        this._rendered = false;
      }
      if (this._rendered) {
        return;
      }
      patterns = {};
      ref = this.elements;
      for (j = 0, len = ref.length; j < len; j++) {
        element = ref[j];
        element.render();
        if (element.pattern != null) {
          baseName = element.pattern.name;
          name = baseName;
          keys = Object.keys(patterns);
          i = 1;
          while (keys.indexOf(name) !== -1) {
            if (element.pattern.isEqualTo(patterns[name])) {
              break;
            } else {
              name = baseName + "-" + (i++);
              element.pattern.name = name;
            }
          }
          patterns[name] = element.pattern;
        }
      }
      return this._rendered = true;
    };

    return QedaLibrary;

  })();

  module.exports = QedaLibrary;

}).call(this);
